/* 
 *   ohlpcustomuifunctions.js
 *   This file contains functions that updates ui behaviors in the OHLP Portal website.
 *   
 *   !important! in order to upload the updated script, please make a copy of the updated file and remove the ".js" file extension. 
 *    the file with js extension can't be imported in Power pgaes Management App.
 * 
 *   Contents of this file:
 *   |no-scroll-in-grid|
 *   |loading-overlay|
 *   |retain-tab-focus|
*/

const localStorgeKeyForBusinessEntity = "activeTab-BusinessEntity";
const localStorgeKeyForAccountLocation = "activeTab-AccountLocation";
const localStorgeKeyForTaxingDistrict = "activeTab-TaxingDistrict";

// *loading-overlay*
function showLoading(message = "Loading, please wait...") {
    const overlay = document.getElementById("loadingOverlay");
    const messageElement = document.getElementById("loadingMessage");
    if (messageElement) {
        messageElement.textContent = message;
    }
    if (overlay) {
        overlay.style.display = "flex";
    }
}

function hideLoading() {
    const overlay = document.getElementById("loadingOverlay");
    if (overlay) {
        overlay.style.display = "none";
    }
}

function clearCurrentTabAtBusinessEntity() {
    localStorage.removeItem(localStorgeKeyForBusinessEntity);
}
function clearCurrentTabAtAccountLocation() {
    localStorage.removeItem(localStorgeKeyForAccountLocation);
}
function clearCurrentTabAtTaxingDistrict() {
    localStorage.removeItem(localStorgeKeyForTaxingDistrict);
}

function convertRadioGroupToVertical(elementId) {
    const $radioButtons = $('#' + elementId + ' input[type="radio"]');

    $radioButtons.each(function () {
        // Create a new div element
        const $div = $('<div></div>');

        // Get the parent node of the radio button
        const $parent = $(this).parent();

        // Get the next sibling of the radio button
        const $nextSibling = $(this).next();

        // Append the radio button and the next sibling to the new div
        $div.append($(this));
        $div.append($nextSibling);

        // Insert both the radio and the sibiling to the new div
        $parent.append($div);
    });
}

//Make mandatory field
function AddRequired(fieldName, isLabelPresent = true, customerrormessage = "") {
    try {
        let validators = Page_Validators.filter(function (el) {
            return el.id == fieldName + "Validator";
        }
        );
        if (validators == undefined || validators == null || validators.length == 0) {
            if ($("#" + fieldName) != undefined) {
                $("#" + fieldName).prop('required', true);
                $("#" + fieldName).closest(".control").prev().addClass("required");
                var Requiredvalidator = document.createElement('span');


                // Create new validator
                var focusField = "javascript:scrollToAndFocus('" + fieldName + "_label','" + fieldName + "');return&nbsp;false;";
                var errorMessage = "";
                var fieldLabelId = "#" + fieldName + "_label";
                if (isLabelPresent) {
                    errorMessage = $("#" + fieldName + "_label").html() + " is a required field.";
                    //console.log("isLabelPresent is true");
                }
                else {
                    fieldLabelId = "#" + fieldName;
                    errorMessage = fieldName + " is a required field.";
                    focusField = "javascript:scrollToAndFocus('" + fieldName + "','" + fieldName + "');return&nbsp;false;";
                    //console.log("isLabelPresent is false");
                }
                if (customerrormessage != "") {
                    errorMessage = customerrormessage;
                    //console.log("Custom error message is not null");
                }

                Requiredvalidator.style.display = "none";
                Requiredvalidator.id = fieldName + "Validator";
                Requiredvalidator.controltovalidate = fieldName;
                Requiredvalidator.errormessage = "<a href='" + fieldLabelId + "' onclick=" + focusField + " referencecontrolid=" + fieldName + ">" + errorMessage + "</a>";
                Requiredvalidator.initialvalue = "";

                if ($("#" + fieldName).is('input[type="file"]')) {
                    Requiredvalidator.evaluationfunction = function () {
                        var fileInput = $("#" + fieldName)[0]; // Get the DOM element
                        if (fileInput.files.length > 0) {
                            return true;
                        }
                        else {
                            return false;
                        }
                    };
                } else {

                    Requiredvalidator.evaluationfunction = function () {
                        var value = $("#" + fieldName).val();
                        if (value == null || value == "") {
                            return false;
                        } else {
                            return true;
                        }
                    };
                }


                // Add the new validator to the page validators array:

                Page_Validators.push(Requiredvalidator);
            }
        }

    }
    catch (error) {
        console.log("Error at AddRequired: " + fieldName);
    }
}
//Make UnMandatory
function RemoveRequired(fieldName) {
    try {
        if ($("#" + fieldName) != undefined) {
            $("#" + fieldName).closest(".control").prev().removeClass("required");
            $("#" + fieldName).prop('required', false);

            for (i = 0; i < Page_Validators.length; i++) {
                if (Page_Validators[i].id == fieldName + "Validator") {
                    Page_Validators.splice(i);
                }
            }
        }
    }
    catch (error) {
        console.log("Error at RemoveRequired: " + fieldName);
    }
}

//Add a validation to check if the value of booth fields match
function AddTwoFieldsMatchValidation(fieldName1, fieldName2, customerrormessage = "") {
    try {
        let validators = Page_Validators.filter(function (el) {
            return el.id == fieldName2 + "MatchValidator";
        }
        );
        if (validators == undefined || validators == null || validators.length == 0) {
            if ($("#" + fieldName) != undefined) {

                var FieldsMatchValidator = document.createElement('span');


                // Create new validator
                var focusField = "javascript:scrollToAndFocus('" + fieldName2 + "_label','" + fieldName2 + "');return&nbsp;false;";
                var errorMessage = "";
                var fieldLabelId = "#" + fieldName2 + "_label";
                errorMessage = "Values on " + $("#" + fieldName1 + "_label").html() + " and " + $("#" + fieldName2 + "_label").html() + " must be present and the same.";

                if (customerrormessage != "") {
                    errorMessage = customerrormessage;
                    //console.log("Custom error message is not null");
                }

                FieldsMatchValidator.style.display = "none";
                FieldsMatchValidator.id = fieldName2 + "MatchValidator";
                FieldsMatchValidator.controltovalidate = fieldName2;
                FieldsMatchValidator.errormessage = "<a href='" + fieldLabelId + "' onclick=" + focusField + " referencecontrolid=" + fieldName + ">" + errorMessage + "</a>";
                FieldsMatchValidator.initialvalue = "";

                FieldsMatchValidator.evaluationfunction = function () {
                    var value1 = $("#" + fieldName1).val();
                    var value2 = $("#" + fieldName2).val();
                    if (value1 != null && value1 != "" && value2 != null && value2 != "" && value1 == value2) {
                        return true;
                    } else {
                        return false;
                    }
                };


                // Add the new validator to the page validators array:

                Page_Validators.push(Requiredvalidator);
            }
        }

    }
    catch (error) {
        console.log("Error at FieldsMatch: " + fieldName);
    }
}

function SetDefaultValueToLookup(fieldSchemaName, guid, name, tableSchemaName) {
    try {
        var lookupField = $("#" + fieldSchemaName);
        if (lookupField) {
            $("#" + fieldSchemaName + "_name").val(name);
            $("#" + fieldSchemaName).val(guid);
            $("#" + fieldSchemaName + "_entityname").val(tableSchemaName);
        }
    }
    catch (error) {
        console.log("Error at SetLookupDefaultValue: " + fieldSchemaName);
    }
}

$(document).ready(function () {
    const queryString = window.location.search;
    const urlParams = new URLSearchParams(queryString);
    // *no-scroll-in-grid*
    // remove scroll bar when there is 1-2 rows in a grid and an action bar appears.
    // removing scroll bar by dynamically changing row height when an action bar pops up.
    $(".entity-grid").on("loaded", function () {
        $(this).children(".view-grid").find("div.dropdown.action").each(function (i, e) {
            var trHeight = $(this).closest("tr").height();
            $(this).on('shown.bs.dropdown', function () {
                $(this).closest("tr").height($(this).height() * 2 + $(this).find("ul.dropdown-menu.show").height());
            });
            $(this).on('hidden.bs.dropdown', function () {
                $(this).closest("tr").height(trHeight);
            });
        });
    });

    // *retain-tab-focus*
    // remove scroll bar when there is 1-2 rows in a grid and an action bar appears.
    // removing scroll bar by dynamically changing row height when an action bar pops up.


    $('#tabs-business-entity button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForBusinessEntity, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });

    $('#tabs-account-location button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForAccountLocation, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });
    $('#tabs-taxing-district button[data-bs-toggle="tab"]').on('click', function (e) {
        try {
            localStorage.setItem(localStorgeKeyForTaxingDistrict, e.target.dataset.bsTarget);
        } catch (e) {
            console.log(e);
        }
    });


    const tabId = urlParams.get('tabId');

    if (tabId) {
        if ($("#tabs-business-entity").length > 0) {
            localStorage.setItem(localStorgeKeyForBusinessEntity, "#" + tabId);
        }

        if ($("#tabs-account-location").length > 0) {
            localStorage.setItem(localStorgeKeyForAccountLocation, "#" + tabId);
        }

        if ($("#tabs-taxing-district").length > 0) {
            localStorage.setItem(localStorgeKeyForTaxingDistrict, "#" + tabId);
        }
    }

    try {
        var activeTabEntity = localStorage.getItem(localStorgeKeyForBusinessEntity);
        var activeTabLocation = localStorage.getItem(localStorgeKeyForAccountLocation);
        var activeTabTaxingDitrict = localStorage.getItem(localStorgeKeyForTaxingDistrict);
    } catch (e) {
        console.log(e);
    }
    if (activeTabEntity) {
        const triggerEl = document.querySelector(`#tabs-business-entity button[data-bs-target="${activeTabEntity}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }
    if (activeTabLocation) {
        const triggerEl = document.querySelector(`#tabs-account-location button[data-bs-target="${activeTabLocation}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }
    if (activeTabTaxingDitrict) {
        const triggerEl = document.querySelector(`#tabs-taxing-district button[data-bs-target="${activeTabTaxingDitrict}"]`);
        if (triggerEl) {
            bootstrap.Tab.getOrCreateInstance(triggerEl).show()
        }
    }


});